USE [igops]
GO

DROP VIEW IF EXISTS review_coverage_users_v;
DROP VIEW IF EXISTS review_coverage_permissions_v;
DROP VIEW IF EXISTS review_coverage_accounts_v;
DROP VIEW IF EXISTS review_coverage_roles_v;
DROP VIEW IF EXISTS review_coverage_broles_v;
GO

CREATE VIEW review_coverage_users_v AS
  SELECT
    suser.display_name,
    suser.display_description,
    suser.risk,
    suser.department
  FROM suser
  WHERE suser.unique_user_id NOT IN (
    SELECT item.unique_user_id
    FROM review_item item
    WHERE item.unique_user_id IS NOT NULL)
    AND suser.effective = 1 AND suser.deleted = 0
GO

CREATE VIEW review_coverage_permissions_v AS
  SELECT
    perm.name,
    perm.description,
    perm.risk,
    app.id app_id,
    app.name app_name
  FROM spermission perm
    JOIN data_source app ON app.id = perm.application
  WHERE perm.unique_permission_id NOT IN (
    SELECT item.unique_permission_id
    FROM review_item item
    WHERE item.unique_permission_id IS NOT NULL)
    AND perm.effective = 1 AND perm.deleted = 0 AND perm.exclude_from_catalog = 0
GO

CREATE VIEW review_coverage_accounts_v AS
  SELECT 
    acc.name,
    acc.description,
    acc.risk,
    app.id app_id,
    app.name app_name
  FROM saccount acc
    JOIN data_source app ON app.id = acc.application
  WHERE acc.unique_account_id NOT IN (
    SELECT item.unique_account_id
    FROM review_item item
    WHERE item.unique_account_id IS NOT NULL)
    AND acc.effective = 1 AND acc.deleted = 0 AND acc.virtual = 0
GO

CREATE VIEW review_coverage_roles_v AS
  SELECT 
    role.name,
    role.description,
    role.risk,
    role_owners.users user_owners,
    role_owners.groups group_owners
  FROM role_policy role
    JOIN (SELECT 
      role_policy.id,
      suser.owners users,
      sgroup.owners groups
      FROM role_policy
        LEFT JOIN (SELECT 
            pol_owner.role_policy, 
            STUFF((SELECT ' | ' + CAST(suser1.display_name AS nvarchar(MAX)) FROM policy_owner pol_owner1
              JOIN suser suser1 on pol_owner1.unique_owner_id = suser1.unique_user_id AND pol_owner1.is_user = 1 AND suser1.effective = 1
              WHERE pol_owner1.role_policy IS NOT NULL AND pol_owner1.role_policy = pol_owner.role_policy
              GROUP BY pol_owner1.role_policy, suser1.display_name
              ORDER BY suser1.display_name FOR XML PATH('')), 1, 3, '') owners
          FROM policy_owner pol_owner
            JOIN suser ON pol_owner.unique_owner_id = suser.unique_user_id AND pol_owner.is_user = 1 AND suser.effective = 1
          WHERE pol_owner.role_policy IS NOT NULL
          GROUP BY pol_owner.role_policy) suser ON suser.role_policy = role_policy.id
        LEFT JOIN (SELECT 
            pol_owner.role_policy, 
            STUFF((SELECT ' | ' + CAST(sgroup1.name AS nvarchar(MAX)) FROM policy_owner pol_owner1
              JOIN sgroup sgroup1 on pol_owner1.unique_owner_id = sgroup1.unique_group_id AND pol_owner1.is_user = 0 AND sgroup1.effective = 1
              WHERE pol_owner1.role_policy IS NOT NULL AND pol_owner1.role_policy = pol_owner.role_policy
              GROUP BY pol_owner1.role_policy, sgroup1.name
              ORDER BY sgroup1.name FOR XML PATH('')), 1, 3, '') owners
          FROM policy_owner pol_owner
            JOIN sgroup ON pol_owner.unique_owner_id = sgroup.unique_group_id AND pol_owner.is_user = 0 AND sgroup.effective = 1
          WHERE pol_owner.role_policy IS NOT NULL
          GROUP BY pol_owner.role_policy) sgroup ON sgroup.role_policy = role_policy.id) role_owners ON role.id = role_owners.id
  WHERE role.id NOT IN (
    SELECT item.role_id
    FROM review_item item
    WHERE item.role_id IS NOT NULL)
    AND role.deleted = 0
GO

CREATE VIEW review_coverage_broles_v AS
  SELECT
    brole.name,
    brole.description,
    brole.risk,
    brole_owners.users user_owners,
    brole_owners.groups group_owners
  FROM business_role brole
  JOIN (SELECT 
      business_role.id,
      suser.owners users,
      sgroup.owners groups
    FROM business_role
    LEFT JOIN (SELECT 
        br_owner.business_role_id, 
        STUFF((SELECT ' | ' + CAST(suser1.display_name AS nvarchar(MAX)) FROM business_role_owner br_owner1
          JOIN suser suser1 on br_owner1.unique_owner_id = suser1.unique_user_id AND br_owner1.is_user = 1 AND suser1.effective = 1
          WHERE br_owner1.business_role_id IS NOT NULL AND br_owner1.business_role_id = br_owner.business_role_id
          GROUP BY br_owner1.business_role_id, suser1.display_name
          ORDER BY suser1.display_name FOR XML PATH('')), 1, 3, '') owners
      FROM business_role_owner br_owner
      JOIN suser ON br_owner.unique_owner_id = suser.unique_user_id AND br_owner.is_user = 1 AND suser.effective = 1
      WHERE br_owner.business_role_id IS NOT NULL
      GROUP BY br_owner.business_role_id) suser ON suser.business_role_id = business_role.id
    LEFT JOIN (SELECT 
        br_owner.business_role_id, 
        STUFF((SELECT ' | ' + CAST(sgroup1.name AS nvarchar(MAX)) FROM business_role_owner br_owner1
          JOIN sgroup sgroup1 on br_owner1.unique_owner_id = sgroup1.unique_group_id AND br_owner1.is_user = 0 AND sgroup1.effective = 1
          WHERE br_owner1.business_role_id IS NOT NULL AND br_owner1.business_role_id = br_owner.business_role_id
          GROUP BY br_owner1.business_role_id, sgroup1.name
          ORDER BY sgroup1.name FOR XML PATH('')), 1, 3, '') owners
      FROM business_role_owner br_owner
      JOIN sgroup ON br_owner.unique_owner_id = sgroup.unique_group_id AND br_owner.is_user = 0 AND sgroup.effective = 1
      WHERE br_owner.business_role_id IS NOT NULL
      GROUP BY br_owner.business_role_id) sgroup ON sgroup.business_role_id = business_role.id) brole_owners ON brole_owners.id = brole.id
  WHERE brole.approval_state = 'PUBLISHED' AND brole.id NOT IN ( 
    SELECT item.business_role_id
    FROM review_item item
    WHERE item.business_role_id IS NOT NULL)
GO

GRANT SELECT ON review_coverage_users_v TO ig_report_role;
GRANT SELECT ON review_coverage_permissions_v TO ig_report_role;
GRANT SELECT ON review_coverage_accounts_v TO ig_report_role;
GRANT SELECT ON review_coverage_roles_v TO ig_report_role;
GRANT SELECT ON review_coverage_broles_v TO ig_report_role;
GO